@@ -21,18 +21,17 @@
 
 BABL_CLASS_DECLARE (conversion);
 
-Babl * babl_conversion (const char *name);
-
-
+const Babl * babl_conversion (const char *name);
 
 /* Signature of functions registered for reference type
  * conversions,
  */
-typedef long (*BablFuncPlane)     (char  *src,
-                                   char  *dst,
-                                   int    src_pitch,
-                                   int    dst_pitch,
-                                   long   n);
+typedef long (*BablFuncPlane)     (const char  *src,
+                                   char        *dst,
+                                   int          src_pitch,
+                                   int          dst_pitch,
+                                   long         n,
+                                   void        *user_data);
 
 
 typedef struct
@@ -48,6 +47,7 @@ BablConversion {
       BablFuncPlane      plane;
       BablFuncPlanar     planar;
     } function;
+  void                  *data;  /* user data */
   int                    processings;
   long                   pixels;
 } BablConversion;
