@@ -24,6 +24,7 @@
 #define NEEDS_BABL_DB
 #include "babl-internal.h"
 #include "babl-db.h"
+#include "babl-ref-pixels.h"
 
 static int babl_format_destruct (void *babl)
 {
@@ -48,7 +49,7 @@ format_new (const char     *name,
             BablModel      *model,
             BablComponent **component,
             BablSampling  **sampling,
-            BablType      **type)
+            const BablType **type)
 {
   Babl *babl;
 
@@ -67,8 +68,10 @@ format_new (const char     *name,
       if (!component_found)
         {
           component_found = 0;
-          babl_fatal ("matching source component for %s in model %s not found",
+#if 0 /* since babl does the right thing, let it happen */
+          babl_fatal("matching source component for %s in model %s not found",
                       model->component[i]->instance.name, model->instance.name);
+#endif
         }
     }
 
@@ -113,23 +116,24 @@ format_new (const char     *name,
   babl->format.visited = 0;
   babl->format.image_template = NULL;
   babl->format.format_n = 0;
+  babl->format.palette = 0;
 
   return babl;
 }
 
 
 static char *
-create_name (BablModel      *model,
-             int             components,
-             BablComponent **component,
-             BablType      **type)
+create_name (const BablModel *model,
+             int              components,
+             BablComponent  **component,
+             const BablType **type)
 {
   char            buf[512] = "";
   char           *p = &buf[0];
   int             i;
   int             same_types = 1;
-  BablType      **t          = type;
-  BablType       *first_type = *type;
+  const BablType**t          = type;
+  const BablType *first_type = *type;
   BablComponent **c1         = component;
   BablComponent **c2         = model->component;
 
@@ -188,8 +192,8 @@ create_name (BablModel      *model,
 
 
 static char *
-ncomponents_create_name (Babl *type,
-                         int   components)
+ncomponents_create_name (const Babl *type,
+                         int         components)
 {
   char buf[512];
   sprintf (buf, "%s[%i] ", type->instance.name, components);
@@ -205,8 +209,8 @@ babl_format_set_is_format_n (Babl *format)
     }
 }
 
-Babl *
-babl_format_n (Babl *btype,
+const Babl *
+babl_format_n (const Babl *btype,
                int   components)
 {
   int            i;
@@ -216,7 +220,7 @@ babl_format_n (Babl *btype,
   BablModel     *model      = (BablModel *)babl_model ("Y");
   BablComponent *component [components];
   BablSampling  *sampling  [components];
-  BablType      *type      [components];
+  const BablType *type      [components];
   char          *name       = NULL;
 
   for (i = 0; i<components; i++)
@@ -250,7 +254,7 @@ babl_format_n (Babl *btype,
 }
 
 int
-babl_format_is_format_n (Babl *format)
+babl_format_is_format_n (const Babl *format)
 {
   if (format->class_type == BABL_FORMAT)
     {
@@ -262,13 +266,13 @@ babl_format_is_format_n (Babl *format)
 
 
 static int
-is_format_duplicate (Babl           *babl,
-                     int             planar,
-                     int             components,
-                     BablModel      *model,
-                     BablComponent **component,
-                     BablSampling  **sampling,
-                     BablType      **type)
+is_format_duplicate (Babl            *babl,
+                     int              planar,
+                     int              components,
+                     BablModel       *model,
+                     BablComponent  **component,
+                     BablSampling   **sampling,
+                     const BablType **type)
 {
   int i;
 
@@ -287,8 +291,8 @@ is_format_duplicate (Babl           *babl,
   return 1;
 }
 
-Babl *
-babl_format_new (void *first_arg,
+const Babl *
+babl_format_new (const void *first_arg,
                  ...)
 {
   va_list        varg;
@@ -299,18 +303,40 @@ babl_format_new (void *first_arg,
   BablModel     *model      = NULL;
   BablComponent *component [BABL_MAX_COMPONENTS];
   BablSampling  *sampling  [BABL_MAX_COMPONENTS];
-  BablType      *type      [BABL_MAX_COMPONENTS];
+  const BablType*type      [BABL_MAX_COMPONENTS];
 
   BablSampling  *current_sampling = (BablSampling *) babl_sampling (1, 1);
   BablType      *current_type     = (BablType *) babl_type_from_id (BABL_DOUBLE);
   char          *name             = NULL;
-  void          *arg              = first_arg;
+  const void    *arg              = first_arg;
 
   va_start (varg, first_arg);
 
   while (1)
     {
-      if (BABL_IS_BABL (arg))
+      /* first, we assume arguments to be strings */
+      if (!strcmp (arg, "id"))
+        {
+          id = va_arg (varg, int);
+        }
+
+      else if (!strcmp (arg, "name"))
+        {
+          name = babl_strdup (va_arg (varg, char *));
+        }
+
+      else if (!strcmp (arg, "packed"))
+        {
+          planar = 0;
+        }
+
+      else if (!strcmp (arg, "planar"))
+        {
+          planar = 1;
+        }
+
+      /* if we didn't point to a known string, we assume argument to be babl */
+      else if (BABL_IS_BABL (arg))
         {
           Babl *babl = (Babl *) arg;
 
@@ -373,26 +399,6 @@ babl_format_new (void *first_arg,
                 break;
             }
         }
-      /* if we didn't point to a babl, we assume arguments to be strings */
-      else if (!strcmp (arg, "id"))
-        {
-          id = va_arg (varg, int);
-        }
-
-      else if (!strcmp (arg, "name"))
-        {
-          name = babl_strdup (va_arg (varg, char *));
-        }
-
-      else if (!strcmp (arg, "packed"))
-        {
-          planar = 0;
-        }
-
-      else if (!strcmp (arg, "planar"))
-        {
-          planar = 1;
-        }
 
       else
         {
@@ -496,7 +502,7 @@ babl_format_get_n_components (const Babl *format)
   return 0;
 }
 
-Babl *
+const Babl *
 babl_format_get_type (const Babl *format,
                       int         component_index)
 {
@@ -510,9 +516,9 @@ babl_format_get_type (const Babl *format,
   return NULL;
 }
 
-Babl *
-babl_format_with_model_as_type (Babl *model,
-                                Babl *type)
+const Babl *
+babl_format_with_model_as_type (const Babl *model,
+                                const Babl *type)
 {
   BablComponent *component[10];
   int            i;
@@ -540,38 +546,22 @@ babl_format_with_model_as_type (Babl *model,
   );
 }
 
-#define test_pixels    256
-
-static double *
-test_create (void)
-{
-  double *test;
-  int     i;
-
-  srandom (20050728);
-
-  test = babl_malloc (sizeof (double) * test_pixels * 4);
-
-  for (i = 0; i < test_pixels * 4; i++)
-    test [i] = (double) random () / RAND_MAX;
-
-  return test;
-}
-
 double
-babl_format_loss (Babl *babl)
+babl_format_loss (const Babl *babl)
 {
   double  loss = 0.0;
-  double *test;
   void   *original;
   double *clipped;
   void   *destination;
   double *transformed;
 
-  Babl   *ref_fmt;
-  Babl   *fmt;
-  Babl   *fish_to;
-  Babl   *fish_from;
+  const Babl *ref_fmt;
+  const Babl *fmt;
+  Babl       *fish_to;
+  Babl       *fish_from;
+
+  const double *test = babl_get_format_test_pixels ();
+  const int     test_pixels = babl_get_num_format_test_pixels ();
 
   ref_fmt = babl_format_new (
     babl_model ("RGBA"),
@@ -589,7 +579,6 @@ babl_format_loss (Babl *babl)
   fish_to   = babl_fish_reference (ref_fmt, fmt);
   fish_from = babl_fish_reference (fmt, ref_fmt);
 
-  test        = test_create ();
   original    = babl_calloc (test_pixels, fmt->format.bytes_per_pixel);
   clipped     = babl_calloc (test_pixels, ref_fmt->format.bytes_per_pixel);
   destination = babl_calloc (test_pixels, fmt->format.bytes_per_pixel);
@@ -611,10 +600,55 @@ babl_format_loss (Babl *babl)
   babl_free (clipped);
   babl_free (destination);
   babl_free (transformed);
-  babl_free (test);
 
-  babl->format.loss = loss;
+  ((Babl*)babl)->format.loss = loss;
   return loss;
 }
 
+
+void *
+babl_get_user_data (const Babl *babl)
+{
+  switch (babl->instance.class_type)
+    {
+      case BABL_MODEL:
+        return babl->model.data;
+      case BABL_FORMAT:
+        return babl->format.model->data;
+      default:
+        babl_fatal ("babl_get_user_data called on non-model/format");
+    }
+  babl_fatal ("eeeek");
+  return NULL;
+}
+
+void
+babl_set_user_data (const Babl *cbabl, void *data)
+{
+  Babl *babl = (Babl*) cbabl;
+  switch (cbabl->instance.class_type)
+    {
+      case BABL_MODEL:
+        babl->model.data = data;
+        break;
+      case BABL_FORMAT:
+        babl->format.model->data = data;
+        break;
+      default:
+        babl_fatal ("babl_set_user_data called on non-model/format");
+    }
+}
+
+const Babl *
+babl_format_get_model (const Babl *format)
+{
+  if (format->class_type == BABL_FORMAT)
+    {
+      return (Babl*)format->format.model;
+    }
+  return NULL;
+}
+
 BABL_CLASS_IMPLEMENT (format)
+
+
